# Makefile for Interpreter Design Kit using gcc

XIDEK = ../../..

SUPPORT_DIR = $(XIDEK)/support/$(SUPPORT_EXT)
LANG_DIR =    $(XIDEK)/$(LANG)/$(LANG_EXT)
DRIVER_DIR =  $(XIDEK)/driver
AGCLIB_DIR =  $(XIDEK)/agclib1
TEST_DIR =    $(XIDEK)/test/$(LANG)

ifndef RES
RES = r2
endif

CXX = g++
CFLAGS = -g -Wall -Wwrite-strings -Wno-unused -DUSE_STD\
	 -I$(SUPPORT_DIR) -I$(LANG_DIR) -I$(AGCLIB_DIR)/include

DRIVER_OBJS = demo.o
SUPPORT_OBJS = comdefs.o bcidefs.o astdefs.o astxi.o astci.o
LANG_OBJS = dxi.o dci.o ast.o

OBJS = $(DRIVER_OBJS) $(SUPPORT_OBJS) $(LANG_OBJS)

SRCS =  $(DRIVER_DIR)/demo.cpp \
	$(LANG_DIR)/dxi.cpp \
	$(LANG_DIR)/dci.cpp \
	$(LANG_DIR)/ast.cpp \
	$(SUPPORT_DIR)/astxi.cpp \
	$(SUPPORT_DIR)/astci.cpp \
	$(SUPPORT_DIR)/comdefs.cpp \
        $(SUPPORT_DIR)/bcidefs.cpp \
        $(SUPPORT_DIR)/astdefs.cpp

DXI_OBJS   = demo.o dxi.o   comdefs.o
DCI_OBJS   = demo.o dci.o   comdefs.o bcidefs.o
ASTXI_OBJS = demo.o astxi.o comdefs.o ast.o astdefs.o
ASTCI_OBJS = demo.o astci.o comdefs.o ast.o astdefs.o bcidefs.o

AGCLIB = $(XIDEK)/agclib1/gcc/agclib1.a

PROGS  = dxi dci astxi astci

DX = t1.dx t2.dx t3.dx $(DX_EXT)
DC = t1.dc t2.dc t3.dc $(DC_EXT)
AX = t1.ax t2.ax t3.ax $(AX_EXT)
AC = t1.ac t2.ac t3.ac $(AC_EXT)

TESTS = $(DX) $(DC) $(AX) $(AC)

# the first rule is the one you get when you type "make" by itself
all: progs tests
progs: $(PROGS)
tests: $(TESTS)

retest: clearTests $(TESTS)

# standard link. $@ is the left side of the rule.
dxi: $(DXI_OBJS) $(AGCLIB)
	$(CXX) -o $@ $(DXI_OBJS) $(AGCLIB)

dci: $(DCI_OBJS) $(AGCLIB)
	$(CXX) -o $@ $(DCI_OBJS) $(AGCLIB)

astxi: $(ASTXI_OBJS) $(AGCLIB)
	$(CXX) -o $@ $(ASTXI_OBJS) $(AGCLIB)

astci: $(ASTCI_OBJS) $(AGCLIB)
	$(CXX) -o $@ $(ASTCI_OBJS) $(AGCLIB)

$(AGCLIB):
	(cd $(XIDEK)/agclib1/gcc && $(MAKE))

#Compile phase
.SUFFIXES: .o .cpp

$(DRIVER_OBJS):
	$(CXX) $(CFLAGS) -c -o $@ $(DRIVER_DIR)/$(@:.o=.cpp)

$(SUPPORT_OBJS):
	$(CXX) $(CFLAGS) -c -o $@ $(SUPPORT_DIR)/$(@:.o=.cpp)

$(LANG_OBJS):
	$(CXX) $(CFLAGS) -c -o $@ $(LANG_DIR)/$(@:.o=.cpp)

#Tests
.SUFFIXES: .dx .dc .ax .ac .scf

#Dependencies
t1.dx: $(TEST_DIR)/t1.scf  dxi
t1.dc: $(TEST_DIR)/t1.scf  dci
t1.ax: $(TEST_DIR)/t1.scf  astxi
t1.ac: $(TEST_DIR)/t1.scf  astci

t2.dx: $(TEST_DIR)/t2.scf  dxi
t2.dc: $(TEST_DIR)/t2.scf  dci
t2.ax: $(TEST_DIR)/t2.scf  astxi
t2.ac: $(TEST_DIR)/t2.scf  astci

t3.dx: $(TEST_DIR)/t3.scf  dxi
t3.dc: $(TEST_DIR)/t3.scf  dci
t3.ax: $(TEST_DIR)/t3.scf  astxi
t3.ac: $(TEST_DIR)/t3.scf  astci

#Rules for running tests
$(DX):
	./dxi $(TEST_DIR)/$(@:.dx=.scf) > $@
	-diff -b $@ $(TEST_DIR)/$(@:.dx=.$(RES))

$(DC):
	./dci $(TEST_DIR)/$(@:.dc=.scf) > $@
	-diff -b $@ $(TEST_DIR)/$(@:.dc=.$(RES))

$(AX):
	./astxi $(TEST_DIR)/$(@:.ax=.scf) > $@
	-diff -b $@ $(TEST_DIR)/$(@:.ax=.$(RES))

$(AC):
	./astci $(TEST_DIR)/$(@:.ac=.scf) > $@
	-diff -b $@ $(TEST_DIR)/$(@:.ac=.$(RES))

# Remove object files and library file for a fresh build
# clean is the standard target name for "remove objects".
clean:	clearTests
	rm -f $(OBJS) $(PROGS) $(TESTS)

clearTests:
	rm -f *.dx *.dc *.ax *.ac

# -MM is gcc for "create header file dependency list"
depend:
	$(CXX) $(CFLAGS) -MM $(SRCS)  > depend.mk

include depend.mk

# Allow gcc to recreate this if it's missing
depend.mk:
	touch depend.mk
	$(MAKE) depend
